#!/bin/sh
# _USE_ psql createdb  chkconfig createuser adduser userdel
# Copyright 2004, Revolution Linux Inc., Benoit des Ligneris
#
# This file is part of the MILLE-XTERM distribution.
# See the MILLE-XTERM (english) and/or the MILLE (french) project web site
#
#	http://www.revolutionlinux.com/mille-xterm/
#	http://www.mille.ca/
#
# The MILLE-XTERM framework is covered by the GNU General Public License.  See
# the COPYING file in the top-level MILLE-XTERM directory.  Software packages
# that are included in the MILLE-XTERM distribution have their own licenses.
#
# -------------------------------------------------------------------------


# This script will install the database for the MILLE-Configurator
# It supposes that postgresql is not secured...

# Make sure that postgresql is installed
if [ ! -f /etc/init.d/postgresql ]; then
	echo 
	echo "Please, install postgresql-server then try again"
	echo 
	exit 0
fi

# Make sure that plpgsql is installed
if [ ! -f /usr/lib/pgsql/plpgsql.so ]; then
        echo
        echo "Please, install postgresql-plpgsql then try again"
        echo
        exit 0
fi



# Add the line to pg_hba.conf if not there
LINE="host    all         all         127.0.0.1         255.255.255.255   trust"
FILE="/var/lib/pgsql/data/pg_hba.conf"
if ! [ "`grep -e "^$LINE" $FILE`" ]
then
	echo "$LINE" >> $FILE
fi

# Modify the line in postgresql.conf
FILE=/var/lib/pgsql/data/postgresql.conf
sed -i -e 's/^#\(tcpip_socket =\).*$/\1\ true/' $FILE
chown postgres.postgres  $FILE

# Add the service
chkconfig --add postgresql

# Restart the service
/etc/init.d/postgresql restart

# Add plpgsql language
su postgres -c "createlang plpgsql template1"

# Create the user
su postgres -c "createuser -d -A mille"
#su postgres -c "createuser -D -A pykotaadmin"
#su postgres -c "createuser -d -A pykotauser"

# Add the mille user
adduser -m mille

# Create the db
su mille -c "createdb  -E 'LATIN1' millev2"


# Make shure we are in the right directory
if [ ! -e './init_DB_Mille_v2.sql' ]; then
	pushd `dirname $0`
fi

# Create the DB and populate the tables
#for i in init_DB_Mille_v2.sql ltsp_attr_DB_Mille.sql data_DB_Mille.sql pykota-postgresql.sql
for i in init_DB_Mille_v2.sql init_DB_Mille_nstree.sql ltsp_attr_DB_Mille.sql mille_attr_DB_Mille.sql data_DB_Mille.sql 
do
su mille -c "psql -U mille millev2 < $i"
done

userdel mille
chkconfig postgresql on
