#!/bin/sh
# _USE_ psql createdb  chkconfig createuser adduser userdel
#!/bin/sh
# Copyright 2006, CRIM, Revolution Linux Inc.
#
# $Id$
#
# Authors : Stephane Bond
#
# This file is part of the MILLE-XTERM project.
#
#       http://www.revolutionlinux.com/mille-xterm/
#       http://www.mille.ca/
#
# This program is covered by the GNU General Public License.
# See the COPYING file in the top-level MILLE-XTERM directory.
#
# -------------------------------------------------------------------------

export PGPASSWORD='config001!'
export OLDMILLEDB='mille'
export NEWMILLEDB='millev2'
export BACKUP_FILE='/tmp/mille_sql_backup.tar'


# Make shure we are in the right directory
if [ ! -e './init_DB_Mille_v2.sql' ]; then
	pushd `dirname $0`
fi

# Make sure that postgresql is installed
if [ ! -f /etc/init.d/postgresql ]; then
        echo
        echo "Please, install postgresql-server then try again"
        echo
        exit 0
fi

# Make sure that plpgsql is installed
if [ ! -f /usr/lib/pgsql/plpgsql.so ]; then
        echo
        echo "Please, install postgresql-plpgsql then try again"
        echo
        exit 0
fi


# Add the mille user
adduser -m mille

# Create a new database with LATIN1 encoding
echo "Create a new database with LATIN1 encoding"
su mille -c "createdb $NEWMILLEDB -U mille -T template1 -E LATIN1"
su postgres -c "createlang -U postgres plpgsql $NEWMILLEDB"

# Backup old database
echo "Backup old database"
su postgres -c "pg_dump -Ft $OLDMILLEDB > $BACKUP_FILE"

# Restore selected tables to the new database
echo "Restore selected tables to the new database"
su mille -c "pg_restore -U mille -t nodes -d $NEWMILLEDB $BACKUP_FILE"
su mille -c "pg_restore -U mille -t attributes -d $NEWMILLEDB $BACKUP_FILE"
su mille -c "pg_restore -U mille -t attributesdef -d $NEWMILLEDB $BACKUP_FILE"
su mille -c "pg_restore -U mille -t attributesselect -d $NEWMILLEDB $BACKUP_FILE"
su mille -c "pg_restore -U mille -t attributesdefdict -d $NEWMILLEDB $BACKUP_FILE"
su mille -c "pg_restore -U mille -t log -d $NEWMILLEDB $BACKUP_FILE"
su mille -c "pg_restore -U mille -t status -d $NEWMILLEDB $BACKUP_FILE"


# Backup old tables and remove serial sequences
echo "Backup old tables and remove serial sequences"
cat << EOF | su mille -c "psql -h localhost $NEWMILLEDB -U mille"
-- Rename tables
ALTER TABLE nodes RENAME TO _back_nodes;
ALTER TABLE attributes RENAME TO _back_attributes;
ALTER TABLE attributesdef RENAME TO _back_attributesdef;
ALTER TABLE attributesselect RENAME TO _back_attributesselect;
ALTER TABLE attributesdefdict RENAME TO _back_attributesdefdict;
ALTER TABLE log RENAME TO _back_log;
ALTER TABLE status RENAME TO _back_status;

-- Remove sequences
ALTER TABLE _back_nodes RENAME id TO id_old;
ALTER TABLE _back_nodes ADD COLUMN id INTEGER;
UPDATE _back_nodes set id = id_old;
ALTER TABLE _back_nodes DROP COLUMN id_old;

ALTER TABLE _back_attributes RENAME id TO id_old;
ALTER TABLE _back_attributes ADD COLUMN id INTEGER;
UPDATE _back_attributes set id = id_old;
ALTER TABLE _back_attributes DROP COLUMN id_old;

ALTER TABLE _back_attributesdef RENAME id TO id_old;
ALTER TABLE _back_attributesdef ADD COLUMN id INTEGER;
UPDATE _back_attributesdef set id = id_old;
ALTER TABLE _back_attributesdef DROP COLUMN id_old;

ALTER TABLE _back_log RENAME id_key TO id_key_old;
ALTER TABLE _back_log ADD COLUMN id_key INTEGER;
UPDATE _back_log set id_key = id_key_old;
ALTER TABLE _back_log DROP COLUMN id_key_old;
EOF


# Add new schema
echo "Add new schema"
su mille -c "psql -U mille $NEWMILLEDB < init_DB_Mille_v2.sql"


# Transfert data
echo "Transfert data"
cat << EOF | su mille -c "psql -h localhost $NEWMILLEDB -U mille"
DELETE FROM nodes;
INSERT INTO nodes (id,id_parent,name,nodetype,mac) SELECT id,id_previous,name,abs(isnode-1),mac FROM _back_nodes;
INSERT INTO attributesdef (id,name,attributeclass,attributetype,mask) SELECT id,name,'ltsp',attributetype,mask FROM _back_attributesdef;
INSERT INTO attributesdefdict (attributesdef_id,value,sortval) SELECT attributesdef_id,value,sortval FROM _back_attributesdefdict;
INSERT INTO attributes (nodes_id,attributesdef_id,value) SELECT nodes_id,attributesdef_id,value FROM _back_attributes;
INSERT INTO attributesselect (nodes_id,attributesdef_id,value) SELECT A.nodes_id,A.attributesdef_id,S.value FROM _back_attributesselect S INNER JOIN _back_attributes A ON A.id=S.attributes_id;
INSERT INTO status (id,mac,ip,bootservip,appservip,display,username,code,bootservretry,appservretry,ts_lastbootservip,ts_lastappservip) SELECT id,mac,ip,bootservip,appservip,display,username,code,bootservretry,appservretry,ts_lastbootservip,ts_lastappservip FROM _back_status;
INSERT INTO log (id,mac,ip,bootservip,appservip,display,username,code,bootservretry,appservretry,ts,id_key) SELECT id,mac,ip,bootservip,appservip,display,username,code,bootservretry,appservretry,ts,id_key FROM _back_log;
EOF


# Add nstree triggers
echo "Add nstree triggers"
su mille -c "psql -U mille $NEWMILLEDB < init_DB_Mille_nstree.sql"


# Update sequences
echo "Update sequences"
cat << EOF | su mille -c "psql -h localhost $NEWMILLEDB -U mille"
SELECT setval('attributesdef_id_seq', max(id)+1) FROM attributesdef;
SELECT setval('log_id_key_seq', max(id_key)+1) FROM log;
SELECT setval('nodes_id_seq', max(id)+1) FROM nodes;
EOF


# Add mille attributes
echo "Add mille attributes"
su mille -c "psql -U mille $NEWMILLEDB < mille_attr_DB_Mille.sql"

# Add remaning attributes
echo "Add remaning attributes"

if [ `echo "select count(*) from attributesDef where name='SMODULE_01'" | su mille -c "psql -h localhost $NEWMILLEDB -U mille" | grep -e "[01]$" | awk {'print $1'}` == "0" ]; then
	echo "INSERT INTO attributesDef (name) VALUES ('SMODULE_01');" | su mille -c "psql -h localhost $NEWMILLEDB -U mille" 
fi

if [ `echo "select count(*) from attributesDef where name='SMODULE_02'" | su mille -c "psql -h localhost $NEWMILLEDB -U mille" | grep -e "[01]$" | awk {'print $1'}` == "0" ]; then
	echo "INSERT INTO attributesDef (name) VALUES ('SMODULE_02');" | su mille -c "psql -h localhost $NEWMILLEDB -U mille"
fi
if [ `echo "select count(*) from attributesDef where name='SMODULE_03'" | su mille -c "psql -h localhost $NEWMILLEDB -U mille" | grep -e "[01]$" | awk {'print $1'}` == "0" ]; then
	echo "INSERT INTO attributesDef (name) VALUES ('SMODULE_03');" | su mille -c "psql -h localhost $NEWMILLEDB -U mille"
fi

if [ `echo "select count(*) from attributesDef where name='SMODULE_04'" | su mille -c "psql -h localhost $NEWMILLEDB -U mille" | grep -e "[01]$" | awk {'print $1'}` == "0" ]; then
	echo "INSERT INTO attributesDef (name) VALUES ('SMODULE_04');" | su mille -c "psql -h localhost $NEWMILLEDB -U mille"
fi

if [ `echo "select count(*) from attributesDef where name='SMODULE_05'" | su mille -c "psql -h localhost $NEWMILLEDB -U mille" | grep -e "[01]$" | awk {'print $1'}` == "0" ]; then
	echo "INSERT INTO attributesDef (name) VALUES ('SMODULE_05');" | su mille -c "psql -h localhost $NEWMILLEDB -U mille"
fi

if [ `echo "select count(*) from attributesDef where name='SMODULE_06'" | su mille -c "psql -h localhost $NEWMILLEDB -U mille" | grep -e "[01]$" | awk {'print $1'}` == "0" ]; then
	echo "INSERT INTO attributesDef (name) VALUES ('SMODULE_06');" | su mille -c "psql -h localhost $NEWMILLEDB -U mille"
fi

if [ `echo "select count(*) from attributesDef where name='SMODULE_07'" | su mille -c "psql -h localhost $NEWMILLEDB -U mille" | grep -e "[01]$" | awk {'print $1'}` == "0" ]; then
	echo "INSERT INTO attributesDef (name) VALUES ('SMODULE_07');" | su mille -c "psql -h localhost $NEWMILLEDB -U mille"
fi

if [ `echo "select count(*) from attributesDef where name='SMODULE_08'" | su mille -c "psql -h localhost $NEWMILLEDB -U mille" | grep -e "[01]$" | awk {'print $1'}` == "0" ]; then
	echo "INSERT INTO attributesDef (name) VALUES ('SMODULE_08');" | su mille -c "psql -h localhost $NEWMILLEDB -U mille"
fi

if [ `echo "select count(*) from attributesDef where name='SMODULE_09'" | su mille -c "psql -h localhost $NEWMILLEDB -U mille" | grep -e "[01]$" | awk {'print $1'}` == "0" ]; then
	echo "INSERT INTO attributesDef (name) VALUES ('SMODULE_09');" | su mille -c "psql -h localhost $NEWMILLEDB -U mille"
fi

if [ `echo "select count(*) from attributesDef where name='SMODULE_10'" | su mille -c "psql -h localhost $NEWMILLEDB -U mille" | grep -e "[01]$" | awk {'print $1'}` == "0" ]; then
	echo "INSERT INTO attributesDef (name) VALUES ('SMODULE_10');" | su mille -c "psql -h localhost $NEWMILLEDB -U mille"
fi

if [ `echo "select count(*) from attributesDef where name='DEFAULT_PRINTER'" | su mille -c "psql -h localhost $NEWMILLEDB -U mille" | grep -e "[01]$" | awk {'print $1'}` == "0" ]; then
	echo "INSERT INTO attributesDef (name) VALUES ('DEFAULT_PRINTER');" | su mille -c "psql -h localhost $NEWMILLEDB -U mille" 
fi


# Remove the mille user
userdel mille

